@@ -127,32 +127,36 @@ from chemicals.heat_capacity import *
 from chemicals.utils import log, exp, isnan
 from chemicals.utils import (to_num, property_molar_to_mass, none_and_length_check,
                           mixing_simple, property_mass_to_molar)
+from chemicals.identifiers import CAS_to_int
 from chemicals import heat_capacity
 from chemicals import miscdata
 from chemicals.miscdata import lookup_VDI_tabular_data
+
 from thermo import electrochem
 from thermo.electrochem import Laliberte_heat_capacity
 from thermo.utils import TDependentProperty, MixtureProperty
 from thermo.coolprop import *
 from cmath import log as clog, exp as cexp
 from thermo.utils import VDI_TABULAR, COOLPROP, POLY_FIT, LINEAR
+from chemicals.miscdata import JOBACK
 
 TRCIG = 'TRCIG'
 POLING_POLY = 'POLING_POLY'
 POLING_CONST = 'POLING_CONST'
 CRCSTD = 'CRCSTD'
 LASTOVKA_SHAW = 'LASTOVKA_SHAW'
-heat_capacity_gas_methods = [COOLPROP, TRCIG, POLING_POLY, LASTOVKA_SHAW, CRCSTD,
-                             POLING_CONST, VDI_TABULAR]
+WEBBOOK_SHOMATE = 'WEBBOOK_SHOMATE'
+heat_capacity_gas_methods = [COOLPROP, TRCIG, WEBBOOK_SHOMATE, POLING_POLY, LASTOVKA_SHAW, CRCSTD,
+                             POLING_CONST, JOBACK, VDI_TABULAR]
 '''Holds all methods available for the :obj:`HeatCapacityGas` class, for use in
 iterating over them.'''
 
 
 class HeatCapacityGas(TDependentProperty):
     r'''Class for dealing with gas heat capacity as a function of temperature.
-    Consists of two coefficient-based methods, two constant methods,
-    one tabular source, one simple estimator, and the external library
-    CoolProp.
+    Consists of three coefficient-based methods, two constant methods,
+    one tabular source, one simple estimator, one group-contribution estimator,
+    and the external library CoolProp.
 
     Parameters
     ----------
@@ -169,12 +173,6 @@ class HeatCapacityGas(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        heat capacity of the compound; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default method if provided. [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -209,14 +207,19 @@ class HeatCapacityGas(TDependentProperty):
     **VDI_TABULAR**:
         Tabular data up to the critical point available in [5]_. Note that this
         data is along the saturation curve.
-
+    **WEBBOOK_SHOMATE**:
+        Shomate form coefficients from [6]_ for ~700 compounds.
+    **JOBACK**:
+        An estimation method for organic substances in [7]_
 
     See Also
     --------
     chemicals.heat_capacity.TRCCp
+    chemicals.heat_capacity.Shomate
     chemicals.heat_capacity.Lastovka_Shaw
     chemicals.heat_capacity.Rowlinson_Poling
     chemicals.heat_capacity.Rowlinson_Bondi
+    thermo.joback.Joback
 
     Examples
     --------
@@ -240,6 +243,12 @@ class HeatCapacityGas(TDependentProperty):
        Chemistry and Physics. [Boca Raton, FL]: CRC press, 2014.
     .. [5] Gesellschaft, V. D. I., ed. VDI Heat Atlas. 2nd edition.
        Berlin; New York:: Springer, 2010.
+    .. [6] Shen, V.K., Siderius, D.W., Krekelberg, W.P., and Hatch, H.W., Eds.,
+       NIST WebBook, NIST, http://doi.org/10.18434/T4M88Q
+    .. [7] Joback, K.G., and R.C. Reid. "Estimation of Pure-Component
+       Properties from Group-Contributions." Chemical Engineering
+       Communications 57, no. 1-6 (July 1, 1987): 233-43.
+       doi:10.1080/00986448708960487.
     '''
     name = 'gas heat capacity'
     units = 'J/mol/K'
@@ -259,7 +268,8 @@ class HeatCapacityGas(TDependentProperty):
     '''Maximum valid of Heat capacity; arbitrarily set. For fluids very near
     the critical point, this value can be obscenely high.'''
 
-    ranked_methods = [TRCIG, POLING_POLY, COOLPROP, LASTOVKA_SHAW, CRCSTD, POLING_CONST, VDI_TABULAR]
+    ranked_methods = [TRCIG, WEBBOOK_SHOMATE, POLING_POLY, COOLPROP, JOBACK,
+                      LASTOVKA_SHAW, CRCSTD, POLING_CONST, VDI_TABULAR]
     '''Default rankings of the available methods.'''
 
 
@@ -271,6 +281,15 @@ class HeatCapacityGas(TDependentProperty):
 
     custom_args = ('MW', 'similarity_variable')
 
+    _json_obj_by_CAS = ('webbook_shomate', 'CP_f')
+    @classmethod
+    def _load_json_CAS_references(cls, d):
+        CASRN = d['CASRN']
+        if CASRN in heat_capacity.WebBook_Shomate_gases:
+            d['webbook_shomate'] = heat_capacity.WebBook_Shomate_gases[CASRN]
+        if 'CP_f' in d:
+            d['CP_f'] = coolprop_fluids[CASRN]
+
     def __init__(self, CASRN='', MW=None, similarity_variable=None,
                  extrapolation='linear', iscyclic_aliphatic=False, **kwargs):
         self.CASRN = CASRN
@@ -293,13 +312,34 @@ class HeatCapacityGas(TDependentProperty):
         methods = []
         self.T_limits = T_limits = {}
         if load_data:
-            if self.CASRN in heat_capacity.TRC_gas_data.index:
+            CASRN = self.CASRN
+            CASRN_int = None if not CASRN else CAS_to_int(CASRN)
+
+            jb_df = miscdata.joback_predictions
+            if CASRN_int in jb_df.index:
+                Cpg3 = float(jb_df.at[CASRN_int, 'Cpg3'])
+                if not isnan(Cpg3):
+                    methods.append(JOBACK)
+                    self.joback_coeffs = [Cpg3,
+                                          float(jb_df.at[CASRN_int, 'Cpg2']),
+                                          float(jb_df.at[CASRN_int, 'Cpg1']),
+                                          float(jb_df.at[CASRN_int, 'Cpg0'])]
+                    Tmin_jb, Tmax_jb = float(jb_df.at[CASRN_int, 'Tm']),float(jb_df.at[CASRN_int, 'Tc'])*2.5
+                    # if isnan(Tmin_jb): Tmin_jb = 100.0 # The same groups are defined for Tm as for Cp, should never be a nan
+                    if isnan(Tmax_jb): Tmax_jb = 10000.0
+                    T_limits[JOBACK] = (Tmin_jb, Tmax_jb)
+
+            if CASRN in heat_capacity.WebBook_Shomate_gases:
+                methods.append(WEBBOOK_SHOMATE)
+                self.webbook_shomate = webbook_shomate = heat_capacity.WebBook_Shomate_gases[CASRN]
+                T_limits[WEBBOOK_SHOMATE] = (webbook_shomate.Tmin, webbook_shomate.Tmax)
+            if CASRN in heat_capacity.TRC_gas_data.index:
                 methods.append(TRCIG)
-                self.TRCIG_Tmin, self.TRCIG_Tmax, a0, a1, a2, a3, a4, a5, a6, a7, _, _, _ = heat_capacity.TRC_gas_values[heat_capacity.TRC_gas_data.index.get_loc(self.CASRN)].tolist()
+                self.TRCIG_Tmin, self.TRCIG_Tmax, a0, a1, a2, a3, a4, a5, a6, a7, _, _, _ = heat_capacity.TRC_gas_values[heat_capacity.TRC_gas_data.index.get_loc(CASRN)].tolist()
                 self.TRCIG_coefs = [a0, a1, a2, a3, a4, a5, a6, a7]
                 T_limits[TRCIG] = (self.TRCIG_Tmin, self.TRCIG_Tmax)
-            if self.CASRN in heat_capacity.Cp_data_Poling.index and not isnan(heat_capacity.Cp_data_Poling.at[self.CASRN, 'a0']):
-                POLING_Tmin, POLING_Tmax, a0, a1, a2, a3, a4, Cpg, Cpl = heat_capacity.Cp_values_Poling[heat_capacity.Cp_data_Poling.index.get_loc(self.CASRN)].tolist()
+            if CASRN in heat_capacity.Cp_data_Poling.index and not isnan(heat_capacity.Cp_data_Poling.at[CASRN, 'a0']):
+                POLING_Tmin, POLING_Tmax, a0, a1, a2, a3, a4, Cpg, Cpl = heat_capacity.Cp_values_Poling[heat_capacity.Cp_data_Poling.index.get_loc(CASRN)].tolist()
                 methods.append(POLING_POLY)
                 if isnan(POLING_Tmin):
                     POLING_Tmin = 50.0
@@ -309,29 +349,29 @@ class HeatCapacityGas(TDependentProperty):
                 self.POLING_Tmax = POLING_Tmax
                 self.POLING_coefs = [a0, a1, a2, a3, a4]
                 T_limits[POLING_POLY] = (POLING_Tmin, POLING_Tmax)
-            if self.CASRN in heat_capacity.Cp_data_Poling.index and not isnan(heat_capacity.Cp_data_Poling.at[self.CASRN, 'Cpg']):
+            if CASRN in heat_capacity.Cp_data_Poling.index and not isnan(heat_capacity.Cp_data_Poling.at[CASRN, 'Cpg']):
                 methods.append(POLING_CONST)
                 self.POLING_T = 298.15
-                self.POLING_constant = float(heat_capacity.Cp_data_Poling.at[self.CASRN, 'Cpg'])
+                self.POLING_constant = float(heat_capacity.Cp_data_Poling.at[CASRN, 'Cpg'])
                 T_limits[POLING_CONST] = (self.POLING_T-50.0, self.POLING_T+50.0)
-            if self.CASRN in heat_capacity.CRC_standard_data.index and not isnan(heat_capacity.CRC_standard_data.at[self.CASRN, 'Cpg']):
+            if CASRN in heat_capacity.CRC_standard_data.index and not isnan(heat_capacity.CRC_standard_data.at[CASRN, 'Cpg']):
                 methods.append(CRCSTD)
                 self.CRCSTD_T = 298.15
-                self.CRCSTD_constant = float(heat_capacity.CRC_standard_data.at[self.CASRN, 'Cpg'])
+                self.CRCSTD_constant = float(heat_capacity.CRC_standard_data.at[CASRN, 'Cpg'])
                 T_limits[CRCSTD] = (self.CRCSTD_T-50.0, self.CRCSTD_T+50.0)
-            if self.CASRN in miscdata.VDI_saturation_dict:
+            if CASRN in miscdata.VDI_saturation_dict:
                 # NOTE: VDI data is for the saturation curve, i.e. at increasing
                 # pressure; it is normally substantially higher than the ideal gas
                 # value
                 methods.append(VDI_TABULAR)
-                Ts, props = lookup_VDI_tabular_data(self.CASRN, 'Cp (g)')
+                Ts, props = lookup_VDI_tabular_data(CASRN, 'Cp (g)')
                 self.VDI_Tmin = Ts[0]
                 self.VDI_Tmax = Ts[-1]
                 self.tabular_data[VDI_TABULAR] = (Ts, props)
                 T_limits[VDI_TABULAR] = (self.VDI_Tmin, self.VDI_Tmax)
-            if has_CoolProp() and self.CASRN in coolprop_dict:
+            if has_CoolProp() and CASRN in coolprop_dict:
                 methods.append(COOLPROP)
-                self.CP_f = coolprop_fluids[self.CASRN]
+                self.CP_f = coolprop_fluids[CASRN]
                 Tmin = max(self.CP_f.Tt, self.CP_f.Tmin)
                 Tmax = min(self.CP_f.Tc, self.CP_f.Tmax)
                 T_limits[COOLPROP] = (Tmin, Tmax)
@@ -339,6 +379,13 @@ class HeatCapacityGas(TDependentProperty):
             methods.append(LASTOVKA_SHAW)
             T_limits[LASTOVKA_SHAW] = (1e-3, 1e5)
         self.all_methods = set(methods)
+        
+    @property
+    def T_limits_fitting(self):
+        values = self.T_limits.copy()
+        if LASTOVKA_SHAW in values:
+            values[LASTOVKA_SHAW] = (150, 3000)
+        return values
 
     @staticmethod
     def _method_indexes():
@@ -373,15 +420,12 @@ class HeatCapacityGas(TDependentProperty):
         Cp : float
             Calculated heat capacity, [J/mol/K]
         '''
-        if method == POLY_FIT:
-            if T < self.poly_fit_Tmin:
-                Cp = (T - self.poly_fit_Tmin)*self.poly_fit_Tmin_slope + self.poly_fit_Tmin_value
-            elif T > self.poly_fit_Tmax:
-                Cp = (T - self.poly_fit_Tmax)*self.poly_fit_Tmax_slope + self.poly_fit_Tmax_value
-            else:
-                Cp = horner(self.poly_fit_coeffs, T)
-        elif method == TRCIG:
+        if method == TRCIG:
             Cp = TRCCp(T, *self.TRCIG_coefs)
+        elif method == WEBBOOK_SHOMATE:
+            Cp = self.webbook_shomate.force_calculate(T)
+        elif method == JOBACK:
+            Cp = horner(self.joback_coeffs, T)
         elif method == COOLPROP:
             try:
                 # Some cases due to melting point need a high pressure
@@ -429,13 +473,7 @@ class HeatCapacityGas(TDependentProperty):
             Whether or not a specifid method is valid
         '''
         validity = True
-        if method == TRCIG:
-            if T < self.TRCIG_Tmin or T > self.TRCIG_Tmax:
-                return False
-        elif method == POLING_POLY:
-            if T < self.POLING_Tmin or T > self.POLING_Tmax:
-                return False
-        elif method == POLING_CONST:
+        if method == POLING_CONST:
             if T > self.POLING_T + 50.0 or T < self.POLING_T - 50.0:
                 return False
         elif method == CRCSTD:
@@ -470,16 +508,12 @@ class HeatCapacityGas(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units*K`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_linear_extrapolation(T1, T2,
-                self.poly_fit_int_coeffs, self.poly_fit_Tmin,
-                self.poly_fit_Tmax, self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == TRCIG:
+        if method == TRCIG:
             H2 = TRCCp_integral(T2, *self.TRCIG_coefs)
             H1 = TRCCp_integral(T1, *self.TRCIG_coefs)
             return H2 - H1
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral(T1, T2)
         elif method == POLING_POLY:
             A, B, C, D, E = self.POLING_coefs
             H2 = (((((0.2*E)*T2 + 0.25*D)*T2 + C/3.)*T2 + 0.5*B)*T2 + A)*T2
@@ -522,17 +556,12 @@ class HeatCapacityGas(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_over_T_linear_extrapolation(T1, T2,
-                self.poly_fit_T_int_T_coeffs, self.poly_fit_log_coeff,
-                self.poly_fit_Tmin, self.poly_fit_Tmax,
-                self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == TRCIG:
+        if method == TRCIG:
             S2 = TRCCp_integral_over_T(T2, *self.TRCIG_coefs)
             S1 = TRCCp_integral_over_T(T1, *self.TRCIG_coefs)
             return S2 - S1
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral_over_T(T1, T2)
         elif method == CRCSTD:
             return self.CRCSTD_constant*log(T2/T1)
         elif method == POLING_CONST:
@@ -569,7 +598,7 @@ DADGOSTAR_SHAW = 'DADGOSTAR_SHAW'
 heat_capacity_liquid_methods = [ZABRANSKY_SPLINE, ZABRANSKY_QUASIPOLYNOMIAL,
                       ZABRANSKY_SPLINE_C, ZABRANSKY_QUASIPOLYNOMIAL_C,
                       ZABRANSKY_SPLINE_SAT, ZABRANSKY_QUASIPOLYNOMIAL_SAT,
-                      VDI_TABULAR, ROWLINSON_POLING, ROWLINSON_BONDI, COOLPROP,
+                      WEBBOOK_SHOMATE, VDI_TABULAR, ROWLINSON_POLING, ROWLINSON_BONDI, COOLPROP,
                       DADGOSTAR_SHAW, POLING_CONST, CRCSTD]
 '''Holds all methods available for the :obj:`HeatCapacityLiquid class`, for use in
 iterating over them.'''
@@ -577,7 +606,7 @@ iterating over them.'''
 
 class HeatCapacityLiquid(TDependentProperty):
     r'''Class for dealing with liquid heat capacity as a function of temperature.
-    Consists of six coefficient-based methods, two constant methods,
+    Consists of seven coefficient-based methods, two constant methods,
     one tabular source, two CSP methods based on gas heat capacity, one simple
     estimator, and the external library CoolProp.
 
@@ -602,12 +631,6 @@ class HeatCapacityLiquid(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        heat capacity of the compound; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default method if provided. [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -679,6 +702,8 @@ class HeatCapacityLiquid(TDependentProperty):
 
         Constant values tabulated in [4]_ at 298.15 K; data is available for 433
         liquids.
+    **WEBBOOK_SHOMATE**:
+        Shomate form coefficients from [6]_ for ~200 compounds.
 
     See Also
     --------
@@ -687,6 +712,7 @@ class HeatCapacityLiquid(TDependentProperty):
     chemicals.heat_capacity.Rowlinson_Poling
     chemicals.heat_capacity.Rowlinson_Bondi
     chemicals.heat_capacity.Dadgostar_Shaw
+    chemicals.heat_capacity.Shomate
 
     Examples
     --------
@@ -708,6 +734,8 @@ class HeatCapacityLiquid(TDependentProperty):
        Chemistry and Physics. [Boca Raton, FL]: CRC press, 2014.
     .. [5] Gesellschaft, V. D. I., ed. VDI Heat Atlas. 2nd edition.
        Berlin; New York:: Springer, 2010.
+    .. [6] Shen, V.K., Siderius, D.W., Krekelberg, W.P., and Hatch, H.W., Eds.,
+       NIST WebBook, NIST, http://doi.org/10.18434/T4M88Q
     '''
     name = 'Liquid heat capacity'
     units = 'J/mol/K'
@@ -732,7 +760,7 @@ class HeatCapacityLiquid(TDependentProperty):
     ranked_methods = [ZABRANSKY_SPLINE, ZABRANSKY_QUASIPOLYNOMIAL,
                       ZABRANSKY_SPLINE_C, ZABRANSKY_QUASIPOLYNOMIAL_C,
                       ZABRANSKY_SPLINE_SAT, ZABRANSKY_QUASIPOLYNOMIAL_SAT,
-                      VDI_TABULAR, COOLPROP, DADGOSTAR_SHAW, ROWLINSON_POLING,
+                      WEBBOOK_SHOMATE, VDI_TABULAR, COOLPROP, DADGOSTAR_SHAW, ROWLINSON_POLING,
                       ROWLINSON_BONDI,
                       POLING_CONST, CRCSTD]
     '''Default rankings of the available methods.'''
@@ -745,7 +773,8 @@ class HeatCapacityLiquid(TDependentProperty):
 
     _json_obj_by_CAS = ('Zabransky_spline', 'Zabransky_spline_iso', 'Zabransky_spline_sat',
                         'Zabransky_quasipolynomial', 'Zabransky_quasipolynomial_iso',
-                        'Zabransky_quasipolynomial_sat', 'CP_f')
+                        'Zabransky_quasipolynomial_sat', 'CP_f',
+                        'webbook_shomate')
 
     obj_references = pure_references = ('Cpgm',)
     obj_references_types = pure_reference_types = (HeatCapacityGas,)
@@ -797,6 +826,8 @@ class HeatCapacityLiquid(TDependentProperty):
             d['Zabransky_quasipolynomial_sat'] = heat_capacity.zabransky_dict_sat_p[CASRN]
         if 'CP_f' in d:
             d['CP_f'] = coolprop_fluids[CASRN]
+        if CASRN in heat_capacity.WebBook_Shomate_liquids:
+            d['webbook_shomate'] = heat_capacity.WebBook_Shomate_liquids[CASRN]
 
     def load_all_methods(self, load_data=True):
         r'''Method which picks out coefficients for the specified chemical
@@ -816,6 +847,10 @@ class HeatCapacityLiquid(TDependentProperty):
                 methods.append(ZABRANSKY_SPLINE)
                 self.Zabransky_spline = heat_capacity.zabransky_dict_const_s[self.CASRN]
                 T_limits[ZABRANSKY_SPLINE] = (self.Zabransky_spline.Tmin, self.Zabransky_spline.Tmax)
+            if self.CASRN in heat_capacity.WebBook_Shomate_liquids:
+                methods.append(WEBBOOK_SHOMATE)
+                self.webbook_shomate = webbook_shomate = heat_capacity.WebBook_Shomate_liquids[self.CASRN]
+                T_limits[WEBBOOK_SHOMATE] = (webbook_shomate.Tmin, webbook_shomate.Tmax)
             if self.CASRN in heat_capacity.zabransky_dict_const_p:
                 methods.append(ZABRANSKY_QUASIPOLYNOMIAL)
                 self.Zabransky_quasipolynomial = heat_capacity.zabransky_dict_const_p[self.CASRN]
@@ -894,14 +929,7 @@ class HeatCapacityLiquid(TDependentProperty):
         Cp : float
             Heat capacity of the liquid at T, [J/mol/K]
         '''
-        if method == POLY_FIT:
-            if T < self.poly_fit_Tmin:
-                return (T - self.poly_fit_Tmin)*self.poly_fit_Tmin_slope + self.poly_fit_Tmin_value
-            elif T > self.poly_fit_Tmax:
-                return (T - self.poly_fit_Tmax)*self.poly_fit_Tmax_slope + self.poly_fit_Tmax_value
-            else:
-                return horner(self.poly_fit_coeffs, T)
-        elif method == ZABRANSKY_SPLINE:
+        if method == ZABRANSKY_SPLINE:
             return self.Zabransky_spline.force_calculate(T)
         elif method == ZABRANSKY_QUASIPOLYNOMIAL:
             return self.Zabransky_quasipolynomial.calculate(T)
@@ -913,6 +941,8 @@ class HeatCapacityLiquid(TDependentProperty):
             return self.Zabransky_spline_sat.force_calculate(T)
         elif method == ZABRANSKY_QUASIPOLYNOMIAL_SAT:
             return self.Zabransky_quasipolynomial_sat.calculate(T)
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate(T)
         elif method == COOLPROP:
             return CoolProp_T_dependent_property(T, self.CASRN , 'CPMOLAR', 'l')
         elif method == POLING_CONST:
@@ -1016,13 +1046,7 @@ class HeatCapacityLiquid(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units*K`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_linear_extrapolation(T1, T2,
-                self.poly_fit_int_coeffs, self.poly_fit_Tmin,
-                self.poly_fit_Tmax, self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == ZABRANSKY_SPLINE:
+        if method == ZABRANSKY_SPLINE:
             return self.Zabransky_spline.calculate_integral(T1, T2)
         elif method == ZABRANSKY_SPLINE_C:
             return self.Zabransky_spline_iso.force_calculate_integral(T1, T2)
@@ -1034,6 +1058,8 @@ class HeatCapacityLiquid(TDependentProperty):
             return self.Zabransky_quasipolynomial_iso.calculate_integral(T1, T2)
         elif method == ZABRANSKY_QUASIPOLYNOMIAL_SAT:
             return self.Zabransky_quasipolynomial_sat.calculate_integral(T1, T2)
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral(T1, T2)
         elif method == POLING_CONST:
             return (T2 - T1)*self.POLING_constant
         elif method == CRCSTD:
@@ -1069,14 +1095,7 @@ class HeatCapacityLiquid(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_over_T_linear_extrapolation(T1, T2,
-                self.poly_fit_T_int_T_coeffs, self.poly_fit_log_coeff,
-                self.poly_fit_Tmin, self.poly_fit_Tmax,
-                self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == ZABRANSKY_SPLINE:
+        if method == ZABRANSKY_SPLINE:
             return self.Zabransky_spline.calculate_integral_over_T(T1, T2)
         elif method == ZABRANSKY_SPLINE_C:
             return self.Zabransky_spline_iso.calculate_integral_over_T(T1, T2)
@@ -1088,6 +1107,8 @@ class HeatCapacityLiquid(TDependentProperty):
             return self.Zabransky_quasipolynomial_iso.calculate_integral_over_T(T1, T2)
         elif method == ZABRANSKY_QUASIPOLYNOMIAL_SAT:
             return self.Zabransky_quasipolynomial_sat.calculate_integral_over_T(T1, T2)
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral_over_T(T1, T2)
         elif method == POLING_CONST:
             return self.POLING_constant*log(T2/T1)
         elif method == CRCSTD:
@@ -1102,14 +1123,14 @@ class HeatCapacityLiquid(TDependentProperty):
 
 LASTOVKA_S = 'LASTOVKA_S'
 PERRY151 = '''PERRY151'''
-heat_capacity_solid_methods = [PERRY151, CRCSTD, LASTOVKA_S]
+heat_capacity_solid_methods = [WEBBOOK_SHOMATE, PERRY151, CRCSTD, LASTOVKA_S]
 '''Holds all methods available for the :obj:`HeatCapacitySolid` class, for use in
 iterating over them.'''
 
 
 class HeatCapacitySolid(TDependentProperty):
     r'''Class for dealing with solid heat capacity as a function of temperature.
-    Consists of one temperature-dependent simple expression, one constant
+    Consists of two temperature-dependent expressions, one constant
     value source, and one simple estimator.
 
     Parameters
@@ -1127,12 +1148,6 @@ class HeatCapacitySolid(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        heat capacity of the compound; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default method if provided. [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -1161,10 +1176,13 @@ class HeatCapacitySolid(TDependentProperty):
         A basic estimation method using the `similarity variable` concept;
         requires only molecular structure, so is very convenient. See
         :obj:`Lastovka_solid <chemicals.heat_capacity.Lastovka_solid>` for details.
+    **WEBBOOK_SHOMATE**:
+        Shomate form coefficients from [3]_ for ~300 compounds.
 
     See Also
     --------
     chemicals.heat_capacity.Lastovka_solid
+    chemicals.heat_capacity.Shomate
 
     Examples
     --------
@@ -1178,6 +1196,8 @@ class HeatCapacitySolid(TDependentProperty):
        Chemistry and Physics. [Boca Raton, FL]: CRC press, 2014.
     .. [2] Green, Don, and Robert Perry. Perry's Chemical Engineers' Handbook,
        Eighth Edition. McGraw-Hill Professional, 2007.
+    .. [3] Shen, V.K., Siderius, D.W., Krekelberg, W.P., and Hatch, H.W., Eds.,
+       NIST WebBook, NIST, http://doi.org/10.18434/T4M88Q
     '''
     name = 'solid heat capacity'
     units = 'J/mol/K'
@@ -1196,7 +1216,7 @@ class HeatCapacitySolid(TDependentProperty):
     property_max = 1E4
     '''Maximum value of Heat capacity; arbitrarily set.'''
 
-    ranked_methods = [PERRY151, CRCSTD, LASTOVKA_S]
+    ranked_methods = [WEBBOOK_SHOMATE, PERRY151, CRCSTD, LASTOVKA_S]
     '''Default rankings of the available methods.'''
 
     _fit_force_n = {}
@@ -1207,6 +1227,14 @@ class HeatCapacitySolid(TDependentProperty):
 
     custom_args = ('MW', 'similarity_variable')
 
+    _json_obj_by_CAS = ('webbook_shomate',)
+
+    @classmethod
+    def _load_json_CAS_references(cls, d):
+        CASRN = d['CASRN']
+        if CASRN in heat_capacity.WebBook_Shomate_solids:
+            d['webbook_shomate'] = heat_capacity.WebBook_Shomate_solids[CASRN]
+
     def __init__(self, CASRN='', similarity_variable=None, MW=None,
                  extrapolation='linear', **kwargs):
         self.similarity_variable = similarity_variable
@@ -1250,6 +1278,10 @@ class HeatCapacitySolid(TDependentProperty):
                 self.CRCSTD_Cp = float(heat_capacity.CRC_standard_data.at[self.CASRN, 'Cps'])
                 methods.append(CRCSTD)
                 T_limits[CRCSTD] = (298.15, 298.15)
+            if self.CASRN in heat_capacity.WebBook_Shomate_solids:
+                methods.append(WEBBOOK_SHOMATE)
+                self.webbook_shomate = webbook_shomate = heat_capacity.WebBook_Shomate_solids[self.CASRN]
+                T_limits[WEBBOOK_SHOMATE] = (webbook_shomate.Tmin, webbook_shomate.Tmax)
         if self.MW and self.similarity_variable:
             methods.append(LASTOVKA_S)
             T_limits[LASTOVKA_S] = (1.0, 1e4)
@@ -1275,14 +1307,7 @@ class HeatCapacitySolid(TDependentProperty):
         Cp : float
             Heat capacity of the solid at T, [J/mol/K]
         '''
-        if method == POLY_FIT:
-            if T < self.poly_fit_Tmin:
-                Cp = (T - self.poly_fit_Tmin)*self.poly_fit_Tmin_slope + self.poly_fit_Tmin_value
-            elif T > self.poly_fit_Tmax:
-                Cp = (T - self.poly_fit_Tmax)*self.poly_fit_Tmax_slope + self.poly_fit_Tmax_value
-            else:
-                Cp = horner(self.poly_fit_coeffs, T)
-        elif method == PERRY151:
+        if method == PERRY151:
             Cp = (self.PERRY151_const + self.PERRY151_lin*T
             + self.PERRY151_quadinv/T**2 + self.PERRY151_quad*T**2)*calorie
         elif method == CRCSTD:
@@ -1290,6 +1315,8 @@ class HeatCapacitySolid(TDependentProperty):
         elif method == LASTOVKA_S:
             Cp = Lastovka_solid(T, self.similarity_variable)
             Cp = property_mass_to_molar(Cp, self.MW)
+        elif method == WEBBOOK_SHOMATE:
+            Cp = self.webbook_shomate.force_calculate(T)
         else:
             return self._base_calculate(T, method)
         return Cp
@@ -1353,13 +1380,7 @@ class HeatCapacitySolid(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units*K`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_linear_extrapolation(T1, T2,
-                self.poly_fit_int_coeffs, self.poly_fit_Tmin,
-                self.poly_fit_Tmax, self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == PERRY151:
+        if method == PERRY151:
             H2 = (self.PERRY151_const*T2 + 0.5*self.PERRY151_lin*T2**2
                   - self.PERRY151_quadinv/T2 + self.PERRY151_quad*T2**3/3.)
             H1 = (self.PERRY151_const*T1 + 0.5*self.PERRY151_lin*T1**2
@@ -1367,6 +1388,8 @@ class HeatCapacitySolid(TDependentProperty):
             return (H2-H1)*calorie
         elif method == CRCSTD:
             return (T2-T1)*self.CRCSTD_Cp
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral(T1, T2)
         elif method == LASTOVKA_S:
             dH = (Lastovka_solid_integral(T2, self.similarity_variable)
                     - Lastovka_solid_integral(T1, self.similarity_variable))
@@ -1394,14 +1417,7 @@ class HeatCapacitySolid(TDependentProperty):
             Calculated integral of the property over the given range,
             [`units`]
         '''
-        if method == POLY_FIT:
-            return fit_integral_over_T_linear_extrapolation(T1, T2,
-                self.poly_fit_T_int_T_coeffs, self.poly_fit_log_coeff,
-                self.poly_fit_Tmin, self.poly_fit_Tmax,
-                self.poly_fit_Tmin_value,
-                self.poly_fit_Tmax_value, self.poly_fit_Tmin_slope,
-                self.poly_fit_Tmax_slope)
-        elif method == PERRY151:
+        if method == PERRY151:
             S2 = (self.PERRY151_const*log(T2) + self.PERRY151_lin*T2
                   - self.PERRY151_quadinv/(2.*T2**2) + 0.5*self.PERRY151_quad*T2**2)
             S1 = (self.PERRY151_const*log(T1) + self.PERRY151_lin*T1
@@ -1415,6 +1431,8 @@ class HeatCapacitySolid(TDependentProperty):
             dS = (Lastovka_solid_integral_over_T(T2, self.similarity_variable)
                     - Lastovka_solid_integral_over_T(T1, self.similarity_variable))
             return property_mass_to_molar(dS, self.MW)
+        elif method == WEBBOOK_SHOMATE:
+            return self.webbook_shomate.force_calculate_integral_over_T(T1, T2)
         else:
             return super(HeatCapacitySolid, self).calculate_integral_over_T(T1, T2, method)
 
