@@ -43,19 +43,22 @@ NRTL Functional Calculations
 ============================
 .. autofunction:: NRTL_gammas
 
+NRTL Regression Calculations
+============================
+.. autofunction:: NRTL_gammas_binaries
 
 '''
 
 from __future__ import division
 from math import log, exp
 from fluids.constants import R
-from fluids.numerics import numpy as np
+from fluids.numerics import numpy as np, trunc_exp
 from thermo.activity import GibbsExcess
 
-__all__ = ['NRTL', 'NRTL_gammas']
+__all__ = ['NRTL', 'NRTL_gammas', 'NRTL_gammas_binaries', 'NRTL_gammas_binaries_jac']
 
 try:
-    array, zeros, npsum, nplog = np.array, np.zeros, np.sum, np.log
+    array, zeros, ones, delete, npsum, nplog = np.array, np.zeros, np.ones, np.delete, np.sum, np.log
 except (ImportError, AttributeError):
     pass
 
@@ -448,6 +451,20 @@ class NRTL(GibbsExcess):
     by the NRTL equation. This model is capable of representing VL and LL
     behavior. [1]_ and [2]_ are good references on this model.
 
+    .. math::
+        g^E = RT\sum_i x_i \frac{\sum_j \tau_{ji} G_{ji} x_j}
+        {\sum_j G_{ji}x_j}
+
+    .. math::
+        G_{ij}=\exp(-\alpha_{ij}\tau_{ij})
+
+    .. math::
+        \alpha_{ij}=c_{ij} + d_{ij}T
+
+    .. math::
+        \tau_{ij}=A_{ij}+\frac{B_{ij}}{T}+E_{ij}\ln T + F_{ij}T
+        + \frac{G_{ij}}{T^2} + H_{ij}{T^2}
+
     Parameters
     ----------
     T : float
@@ -462,24 +479,23 @@ class NRTL(GibbsExcess):
         NRTL alpha parameters, []
     ABEFGHCD : tuple[list[list[float]], 8], optional
         Contains the following. One of (`tau_coeffs` and `alpha_coeffs`) or
-        `ABEFGHCD` are required, [-]
-
-        a : list[list[float]]
-            `a` parameters used in calculating :obj:`NRTL.taus`, [-]
-        b : list[list[float]]
-            `b` parameters used in calculating :obj:`NRTL.taus`, [K]
-        e : list[list[float]]
-            `e` parameters used in calculating :obj:`NRTL.taus`, [-]
-        f : list[list[float]]
-            `f` paraemeters used in calculating :obj:`NRTL.taus`, [1/K]
-        e : list[list[float]]
-            `e` parameters used in calculating :obj:`NRTL.taus`, [K^2]
-        f : list[list[float]]
-            `f` parameters used in calculating :obj:`NRTL.taus`, [1/K^2]
-        c : list[list[float]]
-            `c` parameters used in calculating :obj:`NRTL.alphas`, [-]
-        d : list[list[float]]
-            `d` paraemeters used in calculating :obj:`NRTL.alphas`, [1/K]
+        `ABEFGHCD` or some of the `tau` or `alpha` parameters are required, [-]
+    tau_as : list[list[float]], optional
+        `a` parameters used in calculating :obj:`NRTL.taus`, [-]
+    tau_bs : list[list[float]], optional
+        `b` parameters used in calculating :obj:`NRTL.taus`, [K]
+    tau_es : list[list[float]], optional
+        `e` parameters used in calculating :obj:`NRTL.taus`, [-]
+    tau_fs : list[list[float]], optional
+        `f` paraemeters used in calculating :obj:`NRTL.taus`, [1/K]
+    tau_gs : list[list[float]], optional
+        `e` parameters used in calculating :obj:`NRTL.taus`, [K^2]
+    tau_hs : list[list[float]], optional
+        `f` parameters used in calculating :obj:`NRTL.taus`, [1/K^2]
+    alpha_cs : list[list[float]], optional
+        `c` parameters used in calculating :obj:`NRTL.alphas`, [-]
+    alpha_ds : list[list[float]], optional
+        `d` paraemeters used in calculating :obj:`NRTL.alphas`, [1/K]
 
     Attributes
     ----------
@@ -512,7 +528,7 @@ class NRTL(GibbsExcess):
     >>> GE.gammas()
     [1.93605165145, 1.15366304520]
     >>> GE
-    NRTL(T=343.15, xs=[0.252, 0.748], ABEFGHCD=([[0.0, 0.0], [0.0, 0.0]], [[0, -61.0249799309399], [673.2359767282798, 0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0, 0.2974], [0.2974, 0]], [[0.0, 0.0], [0.0, 0.0]]))
+    NRTL(T=343.15, xs=[0.252, 0.748], tau_bs=[[0, -61.0249799309399], [673.2359767282798, 0]], alpha_cs=[[0, 0.2974], [0.2974, 0]])
     >>> GE.GE(), GE.dGE_dT(), GE.d2GE_dT2()
     (780.053057219, 0.5743500022, -0.003584843605528)
     >>> GE.HE(), GE.SE(), GE.dHE_dT(), GE.dSE_dT()
@@ -531,51 +547,133 @@ class NRTL(GibbsExcess):
        Chemical Thermodynamics for Process Simulation. John Wiley & Sons, 2019.
     '''
     
-    model_attriubtes = ('tau_coeffs_A', 'tau_coeffs_B', 'tau_coeffs_E', 'tau_coeffs_F',
-                         'tau_coeffs_G', 'tau_coeffs_H', 'alpha_coeffs_c', 'alpha_coeffs_d')
+    _model_attributes = ('tau_as', 'tau_bs', 'tau_es', 'tau_fs',
+                         'tau_gs', 'tau_hs', 'alpha_cs', 'alpha_ds')
     model_id = 100
     
     def __init__(self, T, xs, tau_coeffs=None, alpha_coeffs=None,
-                 ABEFGHCD=None):
+                 ABEFGHCD=None, tau_as=None, tau_bs=None, tau_es=None,
+                 tau_fs=None, tau_gs=None, tau_hs=None, alpha_cs=None, 
+                 alpha_ds=None):
         self.T = T
         self.xs = xs
         self.scalar = scalar = type(xs) is list
+        self.N = N = len(xs)
+        
+        multiple_inputs = (tau_as, tau_bs, tau_es, tau_fs, tau_gs, tau_hs, 
+                        alpha_cs, alpha_ds)
+        
+        input_count = ((tau_coeffs is not None or alpha_coeffs is not None) 
+                       + (ABEFGHCD is not None) + (any(i is not None for i in multiple_inputs)))
+        if input_count > 1:
+            raise ValueError("Input only one of (tau_coeffs, alpha_coeffs), ABEFGHCD, or (tau_as...alpha_ds)")
 
-        if ABEFGHCD is not None:
-            (self.tau_coeffs_A, self.tau_coeffs_B, self.tau_coeffs_E,
-            self.tau_coeffs_F, self.tau_coeffs_G, self.tau_coeffs_H,
-            self.alpha_coeffs_c, self.alpha_coeffs_d) = ABEFGHCD
-            self.N = N = len(self.tau_coeffs_A)
+        if ABEFGHCD is None:
+            ABEFGHCD = multiple_inputs
+
+        if tau_coeffs is not None and alpha_coeffs is not None:
+            pass
         else:
-            if tau_coeffs is not None:
-                if scalar:
-                    self.tau_coeffs_A = [[i[0] for i in l] for l in tau_coeffs]
-                    self.tau_coeffs_B = [[i[1] for i in l] for l in tau_coeffs]
-                    self.tau_coeffs_E = [[i[2] for i in l] for l in tau_coeffs]
-                    self.tau_coeffs_F = [[i[3] for i in l] for l in tau_coeffs]
-                    self.tau_coeffs_G = [[i[4] for i in l] for l in tau_coeffs]
-                    self.tau_coeffs_H = [[i[5] for i in l] for l in tau_coeffs]
-                else:
-                    self.tau_coeffs_A = array(tau_coeffs[:,:,0], order='C', copy=True)
-                    self.tau_coeffs_B = array(tau_coeffs[:,:,1], order='C', copy=True)
-                    self.tau_coeffs_E = array(tau_coeffs[:,:,2], order='C', copy=True)
-                    self.tau_coeffs_F = array(tau_coeffs[:,:,3], order='C', copy=True)
-                    self.tau_coeffs_G = array(tau_coeffs[:,:,4], order='C', copy=True)
-                    self.tau_coeffs_H = array(tau_coeffs[:,:,5], order='C', copy=True)
-            else:
-                raise ValueError("`tau_coeffs` is required")
+            try:
+                all_lengths = tuple(len(coeffs) for coeffs in ABEFGHCD if coeffs is not None)
+                if len(set(all_lengths)) > 1:
+                    raise ValueError("Coefficient arrays of different size found: %s" %(all_lengths,))
+                all_lengths_inner = tuple(len(coeffs[0]) for coeffs in ABEFGHCD if coeffs is not None)
+                if len(set(all_lengths_inner)) > 1:
+                    raise ValueError("Coefficient arrays of different size found: %s" %(all_lengths_inner,))
+            except:
+                raise ValueError("Coefficients not input correctly")
 
-            if alpha_coeffs is not None:
-                if scalar:
-                    self.alpha_coeffs_c = [[i[0] for i in l] for l in alpha_coeffs]
-                    self.alpha_coeffs_d = [[i[1] for i in l] for l in alpha_coeffs]
-                else:
-                    self.alpha_coeffs_c = array(alpha_coeffs[:,:,0], order='C', copy=True)
-                    self.alpha_coeffs_d = array(alpha_coeffs[:,:,1], order='C', copy=True)
+        if tau_coeffs is not None and alpha_coeffs is not None:
+            if scalar:
+                self.tau_as = [[i[0] for i in l] for l in tau_coeffs]
+                self.tau_bs = [[i[1] for i in l] for l in tau_coeffs]
+                self.tau_es = [[i[2] for i in l] for l in tau_coeffs]
+                self.tau_fs = [[i[3] for i in l] for l in tau_coeffs]
+                self.tau_gs = [[i[4] for i in l] for l in tau_coeffs]
+                self.tau_hs = [[i[5] for i in l] for l in tau_coeffs]
             else:
-                raise ValueError("`alpha_coeffs` is required")
+                self.tau_as = array(tau_coeffs[:,:,0], order='C', copy=True)
+                self.tau_bs = array(tau_coeffs[:,:,1], order='C', copy=True)
+                self.tau_es = array(tau_coeffs[:,:,2], order='C', copy=True)
+                self.tau_fs = array(tau_coeffs[:,:,3], order='C', copy=True)
+                self.tau_gs = array(tau_coeffs[:,:,4], order='C', copy=True)
+                self.tau_hs = array(tau_coeffs[:,:,5], order='C', copy=True)
+            if scalar:
+                self.alpha_cs = [[i[0] for i in l] for l in alpha_coeffs]
+                self.alpha_ds = [[i[1] for i in l] for l in alpha_coeffs]
+            else:
+                self.alpha_cs = array(alpha_coeffs[:,:,0], order='C', copy=True)
+                self.alpha_ds = array(alpha_coeffs[:,:,1], order='C', copy=True)
+
+        else:
+            if ABEFGHCD[0] is None:
+                self.tau_as = self.zero_coeffs
+            else:
+                self.tau_as = ABEFGHCD[0]
+                
+            if ABEFGHCD[1] is None:
+                self.tau_bs = self.zero_coeffs
+            else:
+                self.tau_bs = ABEFGHCD[1]
+                
+            if ABEFGHCD[2] is None:
+                self.tau_es = self.zero_coeffs
+            else:
+                self.tau_es = ABEFGHCD[2]
+                
+            if ABEFGHCD[3] is None:
+                self.tau_fs = self.zero_coeffs
+            else:
+                self.tau_fs = ABEFGHCD[3]
+                
+            if ABEFGHCD[4] is None:
+                self.tau_gs = self.zero_coeffs
+            else:
+                self.tau_gs = ABEFGHCD[4]
+                
+            if ABEFGHCD[5] is None:
+                self.tau_hs = self.zero_coeffs
+            else:
+                self.tau_hs = ABEFGHCD[5]
+
+            if ABEFGHCD[6] is None:
+                self.alpha_cs = self.zero_coeffs
+            else:
+                self.alpha_cs = ABEFGHCD[6]
+
+            if ABEFGHCD[7] is None:
+                self.alpha_ds = self.zero_coeffs
+            else:
+                self.alpha_ds = ABEFGHCD[7]
+
+        # Make an array of values identifying what coefficients are zero.
+        # This may be useful for performance optimization in the future but is
+        # especially important for reducing the size of the __repr__ string.
+        self.tau_coeffs_nonzero = tau_coeffs_nonzero = [True]*6 if scalar else ones(6, bool)
+        for k, coeffs in enumerate([self.tau_as, self.tau_bs, self.tau_es,
+                           self.tau_fs, self.tau_gs, self.tau_hs]):
+            nonzero = False
+            for i in range(N):
+                r = coeffs[i]
+                for j in range(N):
+                    if r[j] != 0.0:
+                        nonzero = True
+                        break
+                if nonzero:
+                    break
+            tau_coeffs_nonzero[k] = nonzero
+
+
+        alpha_ds = self.alpha_ds
+        alpha_temperature_independent = True
+        for i in range(N):
+            r = alpha_ds[i]
+            for j in range(N):
+                if r[j] != 0.0:
+                    alpha_temperature_independent = False
+        self.alpha_temperature_independent = alpha_temperature_independent
 
-            self.N = N = len(self.tau_coeffs_A)
 
     @property
     def zero_coeffs(self):
@@ -593,10 +691,14 @@ class NRTL(GibbsExcess):
         return self._zero_coeffs
 
     def __repr__(self):
-        s = '%s(T=%s, xs=%s, ABEFGHCD=%s)' %(self.__class__.__name__, repr(self.T), repr(self.xs),
-                (self.tau_coeffs_A,  self.tau_coeffs_B, self.tau_coeffs_E,
-                 self.tau_coeffs_F, self.tau_coeffs_G, self.tau_coeffs_H,
-                 self.alpha_coeffs_c, self.alpha_coeffs_d))
+        s = '%s(T=%s, xs=%s' %(self.__class__.__name__, repr(self.T), repr(self.xs))
+        for i, attr in enumerate(self._model_attributes[:6]):
+            if self.tau_coeffs_nonzero[i]:
+                s += ', %s=%s' %(attr, getattr(self, attr))
+        s += ', alpha_cs=%s' %(self.alpha_cs)        
+        if not self.alpha_temperature_independent:
+            s += ', alpha_ds=%s' %(self.alpha_ds)
+        s += ')'
         return s
 
 
@@ -624,15 +726,13 @@ class NRTL(GibbsExcess):
         have been calculated, they will be set to the new object as well.
         '''
         new = self.__class__.__new__(self.__class__)
-        new.T = T
-        new.xs = xs
-        new.N = self.N
-        new.scalar = self.scalar
-        (new.tau_coeffs_A, new.tau_coeffs_B, new.tau_coeffs_E,
-         new.tau_coeffs_F, new.tau_coeffs_G, new.tau_coeffs_H,
-         new.alpha_coeffs_c, new.alpha_coeffs_d) = (self.tau_coeffs_A, self.tau_coeffs_B, self.tau_coeffs_E,
-                         self.tau_coeffs_F, self.tau_coeffs_G, self.tau_coeffs_H,
-                         self.alpha_coeffs_c, self.alpha_coeffs_d)
+        (new.T, new.xs, new.N, new.scalar) = T, xs, self.N, self.scalar
+        (new.tau_as, new.tau_bs, new.tau_es,
+         new.tau_fs, new.tau_gs, new.tau_hs,
+         new.alpha_cs, new.alpha_ds, new.tau_coeffs_nonzero, 
+         new.alpha_temperature_independent) = (self.tau_as, self.tau_bs, self.tau_es,
+                         self.tau_fs, self.tau_gs, self.tau_hs,
+                         self.alpha_cs, self.alpha_ds, self.tau_coeffs_nonzero, self.alpha_temperature_independent)
 
         if T == self.T:
             try:
@@ -727,12 +827,12 @@ class NRTL(GibbsExcess):
             return self._taus
         except AttributeError:
             pass
-        A = self.tau_coeffs_A
-        B = self.tau_coeffs_B
-        E = self.tau_coeffs_E
-        F = self.tau_coeffs_F
-        G = self.tau_coeffs_G
-        H = self.tau_coeffs_H
+        A = self.tau_as
+        B = self.tau_bs
+        E = self.tau_es
+        F = self.tau_fs
+        G = self.tau_gs
+        H = self.tau_hs
         T, N = self.T, self.N
         if self.scalar:
             taus = [[0.0]*N for _ in range(N)]
@@ -764,11 +864,11 @@ class NRTL(GibbsExcess):
         except AttributeError:
             pass
         # Believed all correct but not tested
-        B = self.tau_coeffs_B
-        E = self.tau_coeffs_E
-        F = self.tau_coeffs_F
-        G = self.tau_coeffs_G
-        H = self.tau_coeffs_H
+        B = self.tau_bs
+        E = self.tau_es
+        F = self.tau_fs
+        G = self.tau_gs
+        H = self.tau_hs
         T, N = self.T, self.N
         if self.scalar:
             dtaus_dT = [[0.0]*N for _ in range(N)]
@@ -799,10 +899,10 @@ class NRTL(GibbsExcess):
             return self._d2taus_dT2
         except AttributeError:
             pass
-        B = self.tau_coeffs_B
-        E = self.tau_coeffs_E
-        G = self.tau_coeffs_G
-        H = self.tau_coeffs_H
+        B = self.tau_bs
+        E = self.tau_es
+        G = self.tau_gs
+        H = self.tau_hs
         T, N = self.T, self.N
 
         if self.scalar:
@@ -834,9 +934,9 @@ class NRTL(GibbsExcess):
             return self._d3taus_dT3
         except AttributeError:
             pass
-        B = self.tau_coeffs_B
-        E = self.tau_coeffs_E
-        G = self.tau_coeffs_G
+        B = self.tau_bs
+        E = self.tau_es
+        G = self.tau_gs
         T, N = self.T, self.N
         if self.scalar:
             d3taus_dT3 = [[0.0]*N for _ in range(N)]
@@ -879,18 +979,21 @@ class NRTL(GibbsExcess):
         except AttributeError:
             pass
         N = self.N
-
-        if self.scalar:
-            alphas = [[0.0]*N for _ in range(N)]
+        
+        if self.alpha_temperature_independent:
+            self._alphas = alphas = self.alpha_cs
         else:
-            alphas = zeros((N, N))
-
-        self._alphas = nrtl_alphas(self.T, N, self.alpha_coeffs_c, self.alpha_coeffs_d, alphas)
+            if self.scalar:
+                alphas = [[0.0]*N for _ in range(N)]
+            else:
+                alphas = zeros((N, N))
+    
+            self._alphas = nrtl_alphas(self.T, N, self.alpha_cs, self.alpha_ds, alphas)
         return alphas
 
     def dalphas_dT(self):
         '''Keep it as a function in case this needs to become more complicated.'''
-        return self.alpha_coeffs_d
+        return self.alpha_ds
 
     def d2alphas_dT2(self):
         '''Keep it as a function in case this needs to become more complicated.'''
@@ -1521,7 +1624,269 @@ class NRTL(GibbsExcess):
         return d2GE_dTdxs
 
 
+    @classmethod
+    def regress_binary_parameters(cls, gammas, xs, symmetric_alphas=False,
+                                  use_numba=False,
+                                  do_statistics=True, **kwargs):
+        # Load the functions either locally or with numba
+        if use_numba:
+            from thermo.numba import NRTL_gammas_binaries as work_func, NRTL_gammas_binaries_jac as jac_func
+        else:
+            work_func = NRTL_gammas_binaries
+            jac_func = NRTL_gammas_binaries_jac
+        
+        # Allocate all working memory
+        pts = len(xs)
+        gammas_iter = zeros(pts*2)
+        jac_iter = zeros((pts*2, 4))
 
+        # Plain objective functions
+        if symmetric_alphas:
+            def fitting_func(xs, tau12, tau21, alpha):
+                return work_func(xs, tau12, tau21, alpha, alpha, gammas_iter)
+            
+            def analytical_jac(xs, tau12, tau21, alpha):
+                return delete(jac_func(xs, tau12, tau21, alpha, alpha, jac_iter), 3, axis=1)
+    
+        else:
+            def fitting_func(xs, tau12, tau21, alpha12, alpha21):
+                return work_func(xs, tau12, tau21, alpha12, alpha21, gammas_iter)
+            
+            def analytical_jac(xs, tau12, tau21, alpha12, alpha21):
+                return jac_func(xs, tau12, tau21, alpha12, alpha21, jac_iter)
+    
+        # The extend calls has been tested to be the fastest compared to numpy and list comprehension
+        xs_working = []
+        for xsi in xs:
+            xs_working.extend(xsi)
+        gammas_working = []
+        for gammasi in gammas:
+            gammas_working.extend(gammasi)
+            
+        xs_working = array(xs_working)
+        gammas_working = array(gammas_working)
+        
+        # Objective functions for leastsq maximum speed
+        if symmetric_alphas:
+            def func_wrapped_for_leastsq(params):
+                return work_func(xs_working, params[0], params[1], params[2], params[2], gammas_iter) - gammas_working
+    
+            def jac_wrapped_for_leastsq(params):
+                out = jac_func(xs_working, params[0], params[1], params[2], params[2], jac_iter)
+                new_out = delete(out, 3, axis=1)
+                new_out[:, 2] = out[:, 2] + out[:, 3]
+                return new_out
+        
+        else:
+            def func_wrapped_for_leastsq(params):
+                return work_func(xs_working, params[0], params[1], params[2], params[3], gammas_iter) - gammas_working
+    
+            def jac_wrapped_for_leastsq(params):
+                return jac_func(xs_working, params[0], params[1], params[2], params[3], jac_iter)
+        
+        if symmetric_alphas:
+            use_fit_parameters = ['tau12', 'tau21', 'alpha12']
+        else:
+            use_fit_parameters = ['tau12', 'tau21', 'alpha12', 'alpha21']
+        return GibbsExcess._regress_binary_parameters(gammas_working, xs_working, fitting_func=fitting_func,
+                                                      fit_parameters=use_fit_parameters,
+                                                      use_fit_parameters=use_fit_parameters,
+                                                      initial_guesses=cls._gamma_parameter_guesses,
+                                                        analytical_jac=jac_func,
+                                                       # analytical_jac=None,
+                                                      use_numba=use_numba,
+                                                      do_statistics=do_statistics,
+                                                      func_wrapped_for_leastsq=func_wrapped_for_leastsq,
+                                                        jac_wrapped_for_leastsq=jac_wrapped_for_leastsq,
+                                                       # jac_wrapped_for_leastsq=None,
+                                                      **kwargs)
+
+    # Larger value on the right always (tau)
+    # Alpha will almost exclusively be fit as one parameter
+    _gamma_parameter_guesses = [{'tau12': 1, 'tau21': 1, 'alpha12': 0.2, 'alpha21': 0.2},
+                                {'tau12': 1, 'tau21': 1, 'alpha12': 0.3, 'alpha21': 0.3},
+                                {'tau12': 1, 'tau21': 1, 'alpha12': 0.47, 'alpha21': 0.47},
+                               ]
+    
+    for i in range(len(_gamma_parameter_guesses)):
+        r = _gamma_parameter_guesses[i]
+        # Swap the taus
+        _gamma_parameter_guesses.append({'tau12': r['tau21'], 'tau21': r['tau12'], 'alpha12': r['alpha12'], 'alpha21': r['alpha21']})
+        # Swap the alphas
+        _gamma_parameter_guesses.append({'tau12': r['tau12'], 'tau21': r['tau21'], 'alpha12': r['alpha21'], 'alpha21': r['alpha12']})
+        # Swap both
+        _gamma_parameter_guesses.append({'tau12': r['tau21'], 'tau21': r['tau12'], 'alpha12': r['alpha21'], 'alpha21': r['alpha12']})
+    del i, r
+
+MIN_TAU_NRTL = -1e100
+MIN_ALPHA_NRTL = 1e-10
+
+def NRTL_gammas_binaries(xs, tau12, tau21, alpha12, alpha21, gammas=None):
+    r'''Calculates activity coefficients at fixed `tau` and `alpha` values for
+    a binary system at a series of mole fractions. This is used for 
+    regression of `tau` and `alpha` parameters. This function is highly optimized,
+    and operates on multiple points at a time.
+    
+    .. math::
+        \ln \gamma_1 = x_2^2\left[\tau_{21}\left(\frac{G_{21}}{x_1 + x_2 G_{21}}
+            \right)^2 + \frac{\tau_{12}G_{12}}{(x_2 + x_1G_{12})^2}
+            \right]
+        
+    .. math::
+        \ln \gamma_2 =  x_1^2\left[\tau_{12}\left(\frac{G_{12}}{x_2 + x_1 G_{12}}
+            \right)^2 + \frac{\tau_{21}G_{21}}{(x_1 + x_2 G_{21})^2}
+            \right]
+        
+    .. math::
+        G_{ij}=\exp(-\alpha_{ij}\tau_{ij})
+        
+
+    Parameters
+    ----------
+    xs : list[float]
+        Liquid mole fractions of each species in the format
+        x0_0, x1_0, (component 1 point1, component 2 point 1),
+        x0_1, x1_1, (component 1 point2, component 2 point 2), ...
+        [-]
+    tau12 : float
+        `tau` parameter for 12, [-]
+    tau21 : float
+        `tau` parameter for 21, [-]
+    alpha12 : float
+        `alpha` parameter for 12, [-]
+    alpha21 : float
+        `alpha` parameter for 21, [-]
+    gammas : list[float], optional
+        Array to store the activity coefficient for each species in the liquid 
+        mixture, indexed the same as `xs`; can be omitted or provided
+        for slightly better performance [-]
+
+    Returns
+    -------
+    gammas : list[float]
+        Activity coefficient for each species in the liquid mixture,
+        indexed the same as `xs`, [-]
+
+    Notes
+    -----
+    
+    Examples
+    --------
+    >>> NRTL_gammas_binaries([.1, .9, 0.3, 0.7, .85, .15], 0.1759, 0.7991, .2, .3)
+    [2.121421, 1.011342, 1.52177, 1.09773, 1.016062, 1.841391]
+    '''
+    if tau12 < MIN_TAU_NRTL:
+        tau12 = MIN_TAU_NRTL
+    if tau21 < MIN_TAU_NRTL:
+        tau21 = MIN_TAU_NRTL
+    if alpha12 < MIN_ALPHA_NRTL:
+        alpha12 = MIN_ALPHA_NRTL
+    if alpha21 < MIN_ALPHA_NRTL:
+        alpha21 = MIN_ALPHA_NRTL
+        
+    pts = len(xs)//2 # Always even
+    
+    if gammas is None:
+        allocate_size = (pts*2)
+        gammas = [0.0]*allocate_size
+        
+    tau01, tau10, alpha01, alpha10 = tau12, tau21, alpha12, alpha21
+
+    G01 = exp(-alpha01*tau01)
+    G10 = exp(-alpha10*tau10)
+    
+    G10_2_tau10 = G10*G10*tau10
+    G10_tau10 = G10*tau10
+
+    G01_2_tau01 = G01*G01*tau01
+    G01_tau01 = G01*tau01
+
+    for i in range(pts):
+        i2 = i*2
+        x0 = xs[i2]
+        x1 = 1.0 - x0
+        
+        c0 = 1.0/(x0 + x1*G10)
+        c0 *= c0
+        
+        c1 = 1.0/(x1 + x0*G01)
+        c1 *= c1
+
+        gamma0 = trunc_exp(x1*x1*(G10_2_tau10*c0 + G01_tau01*c1))
+        gamma1 = trunc_exp(x0*x0*(G01_2_tau01*c1 + G10_tau10*c0))
+
+
+        gammas[i2] = gamma0
+        gammas[i2 + 1] = gamma1
+    return gammas
+
+
+def NRTL_gammas_binaries_jac(xs, tau12, tau21, alpha12, alpha21, calc=None):
+    if tau12 < MIN_TAU_NRTL:
+        tau12 = MIN_TAU_NRTL
+    if tau21 < MIN_TAU_NRTL:
+        tau21 = MIN_TAU_NRTL
+    if alpha12 < MIN_ALPHA_NRTL:
+        alpha12 = MIN_ALPHA_NRTL
+    if alpha21 < MIN_ALPHA_NRTL:
+        alpha21 = MIN_ALPHA_NRTL
+    pts = len(xs)//2 # Always even
+    
+    tau01, tau10, alpha01, alpha10 = tau12, tau21, alpha12, alpha21
+
+    if calc is None:
+        allocate_size = pts*2
+        calc = np.zeros((allocate_size, 4))
+    
+    x2 = alpha01*tau01
+    x3 = trunc_exp(x2)
+    x11 = alpha10*tau10
+    x12 = trunc_exp(x11)
+    x26 = tau01*tau01
+    x27 = tau10*tau10
+    
+    for i in range(pts):
+        i2 = i*2
+        x0 = xs[i2]
+        x1 = 1.0 - x0
+
+
+        x4 = x1*x3
+        x5 = x0 + x4
+        x5_inv = 1.0/x5
+        x6 = 2.0*x4
+        x7 = x6*x5_inv
+        x8 = x2*x7
+        x9 = x5_inv*x5_inv
+        x10 = x1*x1
+        x13 = x0*x12
+        x14 = x1 + x13
+        x14_inv = 1.0/x14
+        x15 = x14_inv*x14_inv
+        x16 = tau10*x15
+        x17 = tau01*x9
+        x18 = x10*trunc_exp(x10*(x16 + x17*x3))
+        x19 = x18*x3*x9
+        x20 = x0*x0
+        x21 = x20*trunc_exp(x20*(x12*x16 + x17))
+        x22 = 2.0*x13
+        x23 = x22*x14_inv
+        x24 = x11*x23
+        x25 = x12*x15*x21
+
+        
+        gamma0_row = calc[i2]
+        gamma0_row[0] = x19*(x2 - x8 + 1.0)# Right
+        gamma0_row[1] = -x15*x18*(x24 - 1.0) # Right
+        gamma0_row[2] = -x19*x26*(x7 - 1.0) # Right
+        gamma0_row[3] =-x18*x22*x27*x15*x14_inv
+        
+        gamma1_row = calc[i2+1]
+        gamma1_row[0] = -x21*x9*(x8 - 1.0)
+        gamma1_row[1] = x25*(x11 - x24 + 1.0)
+        gamma1_row[2] = -x21*x26*x6*x5_inv*x9
+        gamma1_row[3] = -x25*x27*(x23 - 1.0)
+    return calc
 
 def NRTL_gammas(xs, taus, alphas):
     r'''Calculates the activity coefficients of each species in a mixture
