@@ -73,10 +73,11 @@ from chemicals import miscdata
 from chemicals.miscdata import lookup_VDI_tabular_data
 from chemicals.vapor_pressure import *
 from chemicals.vapor_pressure import dAntoine_dT, d2Antoine_dT2, dWagner_original_dT, d2Wagner_original_dT2, dWagner_dT, d2Wagner_dT2, dTRC_Antoine_extended_dT, d2TRC_Antoine_extended_dT2
+from chemicals.identifiers import CAS_to_int
 
 from chemicals import vapor_pressure
 from thermo.utils import TDependentProperty
-from thermo.utils import VDI_TABULAR, DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, EOS, POLY_FIT
+from thermo.utils import VDI_TABULAR, DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, EOS
 from thermo.coolprop import has_CoolProp, PropsSI, coolprop_dict, coolprop_fluids
 from thermo.base import source_path
 
@@ -84,6 +85,7 @@ from thermo.base import source_path
 WAGNER_MCGARRY = 'WAGNER_MCGARRY'
 WAGNER_POLING = 'WAGNER_POLING'
 ANTOINE_POLING = 'ANTOINE_POLING'
+ANTOINE_WEBBOOK = 'ANTOINE_WEBBOOK'
 ANTOINE_EXTENDED_POLING = 'ANTOINE_EXTENDED_POLING'
 
 BOILING_CRITICAL = 'BOILING_CRITICAL'
@@ -91,11 +93,10 @@ LEE_KESLER_PSAT = 'LEE_KESLER_PSAT'
 AMBROSE_WALTON = 'AMBROSE_WALTON'
 SANJARI = 'SANJARI'
 EDALAT = 'EDALAT'
-BEST_FIT_AB = 'POLY_FIT AB extrapolation'
-BEST_FIT_ABC = 'POLY_FIT ABC extrapolation'
 
 vapor_pressure_methods = [WAGNER_MCGARRY, WAGNER_POLING, ANTOINE_EXTENDED_POLING,
-                          DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, ANTOINE_POLING, VDI_TABULAR, AMBROSE_WALTON,
+                          DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, ANTOINE_POLING, VDI_TABULAR, 
+                          ANTOINE_WEBBOOK, AMBROSE_WALTON,
                           LEE_KESLER_PSAT, EDALAT, EOS, BOILING_CRITICAL, SANJARI]
 '''Holds all methods available for the VaporPressure class, for use in
 iterating over them.'''
@@ -103,7 +104,7 @@ iterating over them.'''
 
 class VaporPressure(TDependentProperty):
     '''Class for dealing with vapor pressure as a function of temperature.
-    Consists of four coefficient-based methods and four data sources, one
+    Consists of five coefficient-based methods and four data sources, one
     source of tabular information, four corresponding-states estimators,
     any provided equation of state, and the external library CoolProp.
 
@@ -129,12 +130,6 @@ class VaporPressure(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        vapor pressure of a species; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default method if provided . [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -157,7 +152,11 @@ class VaporPressure(TDependentProperty):
     **ANTOINE_POLING**:
         Standard Antoine equation, as documented in the function
         :obj:`chemicals.vapor_pressure.Antoine` and with data for 325 fluids from [2]_.
-        Coefficients were altered to be in units of Pa and Celcius.
+        Coefficients were altered to be in units of Pa and Kelvin.
+    **ANTOINE_WEBBOOK**:
+        Standard Antoine equation, as documented in the function
+        :obj:`chemicals.vapor_pressure.Antoine` and with data for ~1400 fluids 
+        from [6]_. Coefficients were altered to be in units of Pa and Kelvin.
     **DIPPR_PERRY_8E**:
         A collection of 341 coefficient sets from the DIPPR database published
         openly in [5]_. Provides temperature limits for all its fluids.
@@ -217,6 +216,8 @@ class VaporPressure(TDependentProperty):
        Berlin; New York:: Springer, 2010.
     .. [5] Green, Don, and Robert Perry. Perry's Chemical Engineers' Handbook,
        Eighth Edition. McGraw-Hill Professional, 2007.
+    .. [6] Shen, V.K., Siderius, D.W., Krekelberg, W.P., and Hatch, H.W., Eds.,
+       NIST WebBook, NIST, http://doi.org/10.18434/T4M88Q
     '''
     name = 'Vapor pressure'
     units = 'Pa'
@@ -249,7 +250,8 @@ class VaporPressure(TDependentProperty):
     highest known.'''
 
     ranked_methods = [WAGNER_MCGARRY, WAGNER_POLING, ANTOINE_EXTENDED_POLING,
-                      DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, ANTOINE_POLING, VDI_TABULAR, AMBROSE_WALTON,
+                      DIPPR_PERRY_8E, VDI_PPDS, COOLPROP, ANTOINE_POLING, VDI_TABULAR, 
+                      ANTOINE_WEBBOOK, AMBROSE_WALTON,
                       LEE_KESLER_PSAT, EDALAT, BOILING_CRITICAL, EOS, SANJARI]
     '''Default rankings of the available methods.'''
 
@@ -295,13 +297,22 @@ class VaporPressure(TDependentProperty):
         self.T_limits = T_limits = {}
         methods = []
         if load_data:
-            if self.CASRN in vapor_pressure.Psat_data_WagnerMcGarry.index:
+            CASRN = self.CASRN
+            CASRN_int = None if not CASRN else CAS_to_int(CASRN)
+            df_wb = miscdata.webbook_data
+            if CASRN_int in df_wb.index and not isnan(float(df_wb.at[CASRN_int, 'AntoineA'])):
+                methods.append(ANTOINE_WEBBOOK)
+                self.ANTOINE_WEBBOOK_coefs = [float(df_wb.at[CASRN_int, 'AntoineA']),
+                                              float(df_wb.at[CASRN_int, 'AntoineB']),
+                                              float(df_wb.at[CASRN_int, 'AntoineC'])]
+                T_limits[ANTOINE_WEBBOOK] = (float(df_wb.at[CASRN_int, 'AntoineTmin']),float(df_wb.at[CASRN_int, 'AntoineTmax']))
+            if CASRN in vapor_pressure.Psat_data_WagnerMcGarry.index:
                 methods.append(WAGNER_MCGARRY)
                 A, B, C, D, self.WAGNER_MCGARRY_Pc, self.WAGNER_MCGARRY_Tc, self.WAGNER_MCGARRY_Tmin = vapor_pressure.Psat_values_WagnerMcGarry[vapor_pressure.Psat_data_WagnerMcGarry.index.get_loc(self.CASRN)].tolist()
                 self.WAGNER_MCGARRY_coefs = [A, B, C, D]
                 T_limits[WAGNER_MCGARRY] = (self.WAGNER_MCGARRY_Tmin, self.WAGNER_MCGARRY_Tc)
 
-            if self.CASRN in vapor_pressure.Psat_data_WagnerPoling.index:
+            if CASRN in vapor_pressure.Psat_data_WagnerPoling.index:
                 methods.append(WAGNER_POLING)
                 A, B, C, D, self.WAGNER_POLING_Tc, self.WAGNER_POLING_Pc, Tmin, self.WAGNER_POLING_Tmax = vapor_pressure.Psat_values_WagnerPoling[vapor_pressure.Psat_data_WagnerPoling.index.get_loc(self.CASRN)].tolist()
                 # Some Tmin values are missing; Arbitrary choice of 0.1 lower limit
@@ -310,38 +321,38 @@ class VaporPressure(TDependentProperty):
                 self.WAGNER_POLING_coefs = [A, B, C, D]
                 T_limits[WAGNER_POLING] = (self.WAGNER_POLING_Tmin, self.WAGNER_POLING_Tmax)
 
-            if self.CASRN in vapor_pressure.Psat_data_AntoineExtended.index:
+            if CASRN in vapor_pressure.Psat_data_AntoineExtended.index:
                 methods.append(ANTOINE_EXTENDED_POLING)
                 A, B, C, Tc, to, n, E, F, self.ANTOINE_EXTENDED_POLING_Tmin, self.ANTOINE_EXTENDED_POLING_Tmax = vapor_pressure.Psat_values_AntoineExtended[vapor_pressure.Psat_data_AntoineExtended.index.get_loc(self.CASRN)].tolist()
                 self.ANTOINE_EXTENDED_POLING_coefs = [Tc, to, A, B, C, n, E, F]
                 T_limits[ANTOINE_EXTENDED_POLING] = (self.ANTOINE_EXTENDED_POLING_Tmin, self.ANTOINE_EXTENDED_POLING_Tmax)
 
-            if self.CASRN in vapor_pressure.Psat_data_AntoinePoling.index:
+            if CASRN in vapor_pressure.Psat_data_AntoinePoling.index:
                 methods.append(ANTOINE_POLING)
                 A, B, C, self.ANTOINE_POLING_Tmin, self.ANTOINE_POLING_Tmax = vapor_pressure.Psat_values_AntoinePoling[vapor_pressure.Psat_data_AntoinePoling.index.get_loc(self.CASRN)].tolist()
                 self.ANTOINE_POLING_coefs = [A, B, C]
                 T_limits[ANTOINE_POLING] = (self.ANTOINE_POLING_Tmin, self.ANTOINE_POLING_Tmax)
 
-            if self.CASRN in vapor_pressure.Psat_data_Perrys2_8.index:
+            if CASRN in vapor_pressure.Psat_data_Perrys2_8.index:
                 methods.append(DIPPR_PERRY_8E)
                 C1, C2, C3, C4, C5, self.Perrys2_8_Tmin, self.Perrys2_8_Tmax = vapor_pressure.Psat_values_Perrys2_8[vapor_pressure.Psat_data_Perrys2_8.index.get_loc(self.CASRN)].tolist()
                 self.Perrys2_8_coeffs = [C1, C2, C3, C4, C5]
                 T_limits[DIPPR_PERRY_8E] = (self.Perrys2_8_Tmin, self.Perrys2_8_Tmax)
-            if has_CoolProp() and self.CASRN in coolprop_dict:
+            if has_CoolProp() and CASRN in coolprop_dict:
                 methods.append(COOLPROP)
-                self.CP_f = coolprop_fluids[self.CASRN]
+                self.CP_f = coolprop_fluids[CASRN]
                 T_limits[COOLPROP] = (self.CP_f.Tmin, self.CP_f.Tc)
 
-            if self.CASRN in miscdata.VDI_saturation_dict:
+            if CASRN in miscdata.VDI_saturation_dict:
                 methods.append(VDI_TABULAR)
-                Ts, props = lookup_VDI_tabular_data(self.CASRN, 'P')
+                Ts, props = lookup_VDI_tabular_data(CASRN, 'P')
                 self.VDI_Tmin = Ts[0]
                 self.VDI_Tmax = Ts[-1]
                 self.tabular_data[VDI_TABULAR] = (Ts, props)
                 T_limits[VDI_TABULAR] = (self.VDI_Tmin, self.VDI_Tmax)
 
-            if self.CASRN in vapor_pressure.Psat_data_VDI_PPDS_3.index:
-                Tm, Tc, Pc, A, B, C, D = vapor_pressure.Psat_values_VDI_PPDS_3[vapor_pressure.Psat_data_VDI_PPDS_3.index.get_loc(self.CASRN)].tolist()
+            if CASRN in vapor_pressure.Psat_data_VDI_PPDS_3.index:
+                Tm, Tc, Pc, A, B, C, D = vapor_pressure.Psat_values_VDI_PPDS_3[vapor_pressure.Psat_data_VDI_PPDS_3.index.get_loc(CASRN)].tolist()
                 self.VDI_PPDS_coeffs = [A, B, C, D]
                 self.VDI_PPDS_Tc = Tc
                 self.VDI_PPDS_Tm = Tm
@@ -381,25 +392,7 @@ class VaporPressure(TDependentProperty):
         Psat : float
             Vapor pressure at T, [Pa]
         '''
-        if method == POLY_FIT:
-            if T < self.poly_fit_Tmin:
-                Psat = (T - self.poly_fit_Tmin)*self.poly_fit_Tmin_slope + self.poly_fit_Tmin_value
-            elif T > self.poly_fit_Tmax:
-                Psat = (T - self.poly_fit_Tmax)*self.poly_fit_Tmax_slope + self.poly_fit_Tmax_value
-            else:
-                Psat = horner(self.poly_fit_coeffs, T)
-            Psat = exp(Psat)
-        elif method == BEST_FIT_AB:
-            if T < self.poly_fit_Tmax:
-                return self.calculate(T, POLY_FIT)
-            A, B = self.poly_fit_AB_high_ABC_compat
-            return exp(A + B/T)
-        elif method == BEST_FIT_ABC:
-            if T < self.poly_fit_Tmax:
-                return self.calculate(T, POLY_FIT)
-            A, B, C = self.DIPPR101_ABC_high
-            return exp(A + B/T + C*log(T))
-        elif method == WAGNER_MCGARRY:
+        if method == WAGNER_MCGARRY:
             Psat = Wagner_original(T, self.WAGNER_MCGARRY_Tc, self.WAGNER_MCGARRY_Pc, *self.WAGNER_MCGARRY_coefs)
         elif method == WAGNER_POLING:
             Psat = Wagner(T, self.WAGNER_POLING_Tc, self.WAGNER_POLING_Pc, *self.WAGNER_POLING_coefs)
@@ -408,6 +401,9 @@ class VaporPressure(TDependentProperty):
         elif method == ANTOINE_POLING:
             A, B, C = self.ANTOINE_POLING_coefs
             Psat = Antoine(T, A, B, C, base=10.0)
+        elif method == ANTOINE_WEBBOOK:
+            A, B, C = self.ANTOINE_WEBBOOK_coefs
+            Psat = Antoine(T, A, B, C, base=e)
         elif method == DIPPR_PERRY_8E:
             Psat = EQ101(T, *self.Perrys2_8_coeffs)
         elif method == VDI_PPDS:
@@ -426,8 +422,6 @@ class VaporPressure(TDependentProperty):
             Psat = Edalat(T, self.Tc, self.Pc, self.omega)
         elif method == EOS:
             Psat = self.eos[0].Psat(T)
-        elif method == POLY_FIT:
-            Psat = exp(horner(self.poly_fit_coeffs, T))
         else:
             return self._base_calculate(T, method)
         return Psat
@@ -459,17 +453,8 @@ class VaporPressure(TDependentProperty):
         if method in T_limits:
             Tmin, Tmax = T_limits[method]
             return Tmin <= T <= Tmax
-        elif method == POLY_FIT:
-            return True
-        elif method in self.tabular_data:
-            if self.tabular_extrapolation_permitted:
-                # good to go without checking
-                return True
-            else:
-                Ts, properties = self.tabular_data[method]
-                return Ts[0] < T < Ts[-1]
         else:
-            raise ValueError("invalid method '%s'" %method)
+            return super(VaporPressure, self).test_method_validity(T, method)
 
     def calculate_derivative(self, T, method, order=1):
         r'''Method to calculate a derivative of a vapor pressure with respect to
@@ -496,10 +481,7 @@ class VaporPressure(TDependentProperty):
             Calculated derivative property, [`units/K^order`]
         '''
         Tmin, Tmax = self.T_limits[method]
-        if order == 1 and method == POLY_FIT:
-            v, der = horner_and_der(self.poly_fit_coeffs, T)
-            return der*exp(v)
-        elif method == WAGNER_MCGARRY:
+        if method == WAGNER_MCGARRY:
             if Tmin <= T <= Tmax:
                 if order == 1:
                     return dWagner_original_dT(T, self.WAGNER_MCGARRY_Tc, self.WAGNER_MCGARRY_Pc, *self.WAGNER_MCGARRY_coefs)
@@ -539,81 +521,8 @@ class VaporPressure(TDependentProperty):
                     return EQ101(T, *self.Perrys2_8_coeffs, order=1)
                 if order == 2:
                     return EQ101(T, *self.Perrys2_8_coeffs, order=2)
-        elif order == 2 and method == POLY_FIT:
-            v, der, der2 = horner_and_der2(self.poly_fit_coeffs, T)
-            return (der*der + der2)*exp(v)
         return super(VaporPressure, self).calculate_derivative(T, method, order)
 
-    def _custom_set_poly_fit(self):
-        try:
-            Tmin, Tmax = self.poly_fit_Tmin, self.poly_fit_Tmax
-            poly_fit_coeffs = self.poly_fit_coeffs
-            v_Tmin = horner(poly_fit_coeffs, Tmin)
-            for T_trans in linspace(Tmin, Tmax, 25):
-                v, d1, d2 = horner_and_der2(poly_fit_coeffs, T_trans)
-                Psat = exp(v)
-                dPsat_dT = Psat*d1
-                d2Psat_dT2 = Psat*(d1*d1 + d2)
-
-                A, B, C = Antoine_ABC = Antoine_coeffs_from_point(T_trans, Psat, dPsat_dT, d2Psat_dT2, base=e)
-                self.poly_fit_AB = list(Antoine_AB_coeffs_from_point(T_trans, Psat, dPsat_dT, base=e))
-                self.DIPPR101_ABC = list(DIPPR101_ABC_coeffs_from_point(T_trans, Psat, dPsat_dT, d2Psat_dT2))
-
-                B_OK = B > 0.0 # B is negated in this implementation, so the requirement is reversed
-                C_OK = -T_trans < C < 0.0
-                if B_OK and C_OK:
-                    self.poly_fit_Antoine = Antoine_ABC
-                    break
-                else:
-                    continue
-
-            # Calculate the extrapolation values
-            v_Tmax = horner(poly_fit_coeffs, Tmax)
-            v, d1, d2 = horner_and_der2(poly_fit_coeffs, Tmax)
-            Psat = exp(v)
-            dPsat_dT = Psat*d1
-            d2Psat_dT2 = Psat*(d1*d1 + d2)
-#                A, B, C = Antoine_ABC = Antoine_coeffs_from_point(T_trans, Psat, dPsat_dT, d2Psat_dT2, base=e)
-            self.poly_fit_AB_high = list(Antoine_AB_coeffs_from_point(Tmax, Psat, dPsat_dT, base=e))
-            self.poly_fit_AB_high_ABC_compat = [self.poly_fit_AB_high[0], -self.poly_fit_AB_high[1]]
-            self.DIPPR101_ABC_high = list(DIPPR101_ABC_coeffs_from_point(Tmax, Psat, dPsat_dT, d2Psat_dT2))
-
-
-        except:
-            pass
-
-    def solve_prop_poly_fit(self, goal):
-        poly_fit_Tmin, poly_fit_Tmax = self.poly_fit_Tmin, self.poly_fit_Tmax
-        poly_fit_Tmin_slope, poly_fit_Tmax_slope = self.poly_fit_Tmin_slope, self.poly_fit_Tmax_slope
-        poly_fit_Tmin_value, poly_fit_Tmax_value = self.poly_fit_Tmin_value, self.poly_fit_Tmax_value
-        coeffs = self.poly_fit_coeffs
-
-        T_low = log(goal*exp(poly_fit_Tmin*poly_fit_Tmin_slope - poly_fit_Tmin_value))/poly_fit_Tmin_slope
-        if T_low <= poly_fit_Tmin:
-            return T_low
-        T_high = log(goal*exp(poly_fit_Tmax*poly_fit_Tmax_slope - poly_fit_Tmax_value))/poly_fit_Tmax_slope
-        if T_high >= poly_fit_Tmax:
-            return T_high
-        else:
-            lnPGoal = log(goal)
-            def to_solve(T):
-                # dPsat and Psat are both in log basis
-                dPsat = Psat = 0.0
-                for c in coeffs:
-                    dPsat = T*dPsat + Psat
-                    Psat = T*Psat + c
-
-                return Psat - lnPGoal, dPsat
-            # Guess with the two extrapolations from the linear fits
-            # By definition both guesses are in the range of they would have been returned
-            if T_low > poly_fit_Tmax:
-                T_low = poly_fit_Tmax
-            if T_high < poly_fit_Tmin:
-                T_high = poly_fit_Tmin
-            T = newton(to_solve, 0.5*(T_low + T_high), fprime=True, low=poly_fit_Tmin, high=poly_fit_Tmax)
-            return T
-
-
 PSUB_CLAPEYRON = 'PSUB_CLAPEYRON'
 
 sublimation_pressure_methods = [PSUB_CLAPEYRON]
@@ -643,12 +552,6 @@ class SublimationPressure(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        vapor pressure of a species; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default method if provided . [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -694,7 +597,7 @@ class SublimationPressure(TDependentProperty):
     custom_args = ('Tt', 'Pt', 'Hsub_t')
 
     def __init__(self, CASRN=None, Tt=None, Pt=None, Hsub_t=None,
-                 extrapolation=None, **kwargs):
+                 extrapolation='linear', **kwargs):
         self.CASRN = CASRN
         self.Tt = Tt
         self.Pt = Pt
@@ -746,15 +649,7 @@ class SublimationPressure(TDependentProperty):
         Psub : float
             Sublimation pressure at T, [Pa]
         '''
-        if method == POLY_FIT:
-            if T < self.poly_fit_Tmin:
-                Psub = (T - self.poly_fit_Tmin)*self.poly_fit_Tmin_slope + self.poly_fit_Tmin_value
-            elif T > self.poly_fit_Tmax:
-                Psub = (T - self.poly_fit_Tmax)*self.poly_fit_Tmax_slope + self.poly_fit_Tmax_value
-            else:
-                Psub = horner(self.poly_fit_coeffs, T)
-            Psub = exp(Psub)
-        elif method == PSUB_CLAPEYRON:
+        if method == PSUB_CLAPEYRON:
             Psub = max(Psub_Clapeyron(T, Tt=self.Tt, Pt=self.Pt, Hsub_t=self.Hsub_t), 1e-200)
         else:
             return self._base_calculate(T, method)
@@ -786,19 +681,5 @@ class SublimationPressure(TDependentProperty):
         if method == PSUB_CLAPEYRON:
             return True
             # No lower limit
-        elif method == POLY_FIT:
-            validity = True
-        elif method in self.tabular_data:
-            # if tabular_extrapolation_permitted, good to go without checking
-            if not self.tabular_extrapolation_permitted:
-                Ts, properties = self.tabular_data[method]
-                if T < Ts[0] or T > Ts[-1]:
-                    return False
         else:
-            raise Exception('Method not valid')
-        return True
-try:
-    SublimationPressure._custom_set_poly_fit = VaporPressure._custom_set_poly_fit
-except:
-    pass
-
+            return super(VaporPressure, self).test_method_validity(T, method)
